// SubAtomic.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include <windowsx.h>
#include <mmsystem.h>
#include <stdio.h>
#include <string.h>
#include "Particle.h"

#define WINDOW_CLASS_NAME "WINCLASS1"

HINSTANCE hinstanceApp = NULL;
HWND mainWindowHandle = NULL;
bool windowClosed = false;
HDC myHDC = NULL;
HGLRC myHGLRC = NULL;
int ScreenActive = 0;
#define MAXNUMPARTICLES 1000
Particle *p;
int numParticles = 100;

static double screenX = 1024.0;
static double screenY = 768.0;


BOOL SetupPixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd = 
	{ 
		sizeof(PIXELFORMATDESCRIPTOR),  //  size of this pfd 
			1,                     // version number 
			PFD_DRAW_TO_WINDOW |   // support window 
			PFD_SUPPORT_OPENGL |   // support OpenGL 
			PFD_DOUBLEBUFFER,      // double buffered 
			PFD_TYPE_RGBA,         // RGBA type 
			24,                    // 24-bit color depth 
			0, 0, 0, 0, 0, 0,      // color bits ignored 
			0,                     // no alpha buffer 
			0,                     // shift bit ignored 
			0,                     // no accumulation buffer 
			0, 0, 0, 0,            // accum bits ignored 
			0,                     // 0-bit z-buffer     
			0,                     // no stencil buffer 
			0,                     // no auxiliary buffer 
			PFD_MAIN_PLANE,        // main layer     0,                     // reserved 
			0, 0, 0                // layer masks ignored     
	};

	int pixelFormat = 0;
	if (!(pixelFormat = ::ChoosePixelFormat(myHDC,&pfd)))
	{
		return FALSE;
	}
	if (!::SetPixelFormat(myHDC,pixelFormat,&pfd))
	{
		return FALSE;
	}

	return TRUE;
}

BOOL InitializeOpenGL()
{
	myHDC = GetDC(mainWindowHandle);

	if (!SetupPixelFormat())
	{
		return FALSE;
	}
	if (!(myHGLRC = wglCreateContext(myHDC)))
	{
		return FALSE;
	}
	if (!wglMakeCurrent(myHDC,myHGLRC))
	{
		return FALSE;
	}

	// setup the defaults for OpenGL
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_SCISSOR_TEST);
	glShadeModel(GL_SMOOTH);
//	glDisable(GL_DITHER);
	glDisable(GL_LIGHTING);
//	glEnable(GL_LIGHT0);
	glDisable(GL_CULL_FACE);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glEnable(GL_BLEND);
	glViewport(0,0,(int)screenX,(int)screenY);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(-0.5,screenX+0.5,-0.5,screenY+0.5);
	glMatrixMode(GL_MODELVIEW);
	glClearColor(1.0f,0.125f,0.125f,1.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	SwapBuffers(myHDC);
	glFinish();
	glClearColor(0.125f,0.125f,1.0f,1.0f);
	glClear(GL_COLOR_BUFFER_BIT);
	SwapBuffers(myHDC);
	glFinish();
	
	return TRUE;
}

void OrthoView()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(-0.5,screenX+0.5,-0.5,screenY+0.5);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glDisable(GL_DEPTH_TEST);
}

void PersView()
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,screenX/screenY,10.0f,100000.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-1000.0f);
}

double sys_glWidth = screenX;
double sys_glHeight = screenY;
float particleColor[(MAXNUMPARTICLES+4)*4*4];
float particleVertices[(MAXNUMPARTICLES+4)*2*4];
float particleTextures[(MAXNUMPARTICLES+4)*2*4];
int particleColorIndex;
int particleVerticesIndex;
int particleTexturesIndex;

GLubyte smallTextureArray[32][32];
GLubyte bigTextureArray[256][256][2];
GLuint theTexture = 0;

inline float FastDistance2D(float x, float y)
{
	// this function computes the distance from 0,0 to x,y with ~3.5% error
	
	// first compute the absolute value of x,y
	x = (x < 0.0f) ? -x : x;
	y = (y < 0.0f) ? -y : y;
	
	// compute the minimum of x,y
	float mn = min(x,y);
	
	// return the distance
	return(x+y-(mn*0.5f)-(mn*0.25f)+(mn*0.0625f));
	
} // end FastDistance2D


// simple smoothing routine
void SmoothTexture()
{
	GLubyte filter[32][32];
	int i,j;
	float t;
	for (i=1;i<31;i++)
	{
		for (j=1;j<31;j++)
		{
			t = (float) smallTextureArray[i][j]*4;
			t += (float) smallTextureArray[i-1][j];
			t += (float) smallTextureArray[i+1][j];
			t += (float) smallTextureArray[i][j-1];
			t += (float) smallTextureArray[i][j+1];
			t /= 8.0f;
			filter[i][j] = (GLubyte) t;
		}
	}
	for (i=1;i<31;i++)
	{
		for (j=1;j<31;j++)
		{
			smallTextureArray[i][j] = filter[i][j];
		}
	}
}

// add some randomness to texture data
void SpeckleTexture()
{
	int i,j;
	int speck;
	float t;
	for (i=2;i<30;i++)
	{
		for (j=2;j<30;j++)
		{
			speck = 1;
			while (speck <= 32 && rand() % 2)
			{
				t = (float) min(255,smallTextureArray[i][j]+speck);
				smallTextureArray[i][j] = (GLubyte) t;
				speck+=speck;
			}
			speck = 1;
			while (speck <= 32 && rand() % 2)
			{
				t = (float) max(0,smallTextureArray[i][j]-speck);
				smallTextureArray[i][j] = (GLubyte) t;
				speck+=speck;
			}
		}
	}
}

void MakeSmallTexture()
{
	static int firstTime = 1;
	int i,j;
	float r,t;
	if (firstTime)
	{
		firstTime = 0;
		for (i=0;i<32;i++)
		{
			for (j=0;j<32;j++)
			{
				r = (float) sqrt((i-15.5)*(i-15.5)+(j-15.5)*(j-15.5));
				if (r > 15.0f)
				{
					smallTextureArray[i][j] = 0;
				}
				else
				{
					t = 255.0f * (float) cos(r*PI/31.0);
					smallTextureArray[i][j] = (GLubyte) t;
				}
			}
		}
	}
	else
	{
		for (i=0;i<32;i++)
		{
			for (j=0;j<32;j++)
			{
				r = (float) sqrt((i-15.5)*(i-15.5)+(j-15.5)*(j-15.5));
				if (r > 15.0f)
				{
					t = 0.0f;
				}
				else
				{
					t = 255.0f * (float) cos(r*PI/31.0);
				}
				smallTextureArray[i][j] = (GLubyte) min(255,(t+smallTextureArray[i][j]+smallTextureArray[i][j])/3);
			}
		}
	}
//	SpeckleTexture();
//	SmoothTexture();
//	SmoothTexture();
}

void CopySmallTextureToBigTexture(int k, int l)
{
	int i,j;
	for (i=0;i<32;i++)
	{
		for (j=0;j<32;j++)
		{
			bigTextureArray[i+k][j+l][0] = smallTextureArray[i][j];
			bigTextureArray[i+k][j+l][1] = smallTextureArray[i][j];
		}
	}
}

void AverageLastAndFirstTextures()
{
	int i,j;
	int t;
	for (i=0;i<32;i++)
	{
		for (j=0;j<32;j++)
		{
			t = (smallTextureArray[i][j] + bigTextureArray[i][j][0]) / 2;
			smallTextureArray[i][j] = (GLubyte) min(255,t);
		}
	}
}

void MakeTexture()
{
	int i,j;
	for (i=0;i<8;i++)
	{
		for (j=0;j<8;j++)
		{
			if (i==7 && j==7)
			{
				AverageLastAndFirstTextures();
			}
			else
			{
				MakeSmallTexture();
			}
			CopySmallTextureToBigTexture(i*32,j*32);
		}
	}

	glPixelStorei(GL_UNPACK_ALIGNMENT,1);

	glGenTextures(1, &theTexture); 	
	glBindTexture(GL_TEXTURE_2D, theTexture);
	
	// Set the tiling mode (this is generally always GL_REPEAT).
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	
	// Set the filtering.
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	
	gluBuild2DMipmaps(GL_TEXTURE_2D, 2, 256, 256, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, bigTextureArray);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

int numParticlesDrawn = 0;
void particleListDraw()
{
	numParticlesDrawn = 0;
	particleTexturesIndex = 0;
	particleVerticesIndex = 0;
	particleColorIndex = 0;
	
	Particle *thisParticle; 
	for (int i = 1; i < numParticles; i++)
	{
		thisParticle = &(p[i]);

		if (thisParticle->drawThisFrame)
		{
//			thisParticle->drawThisFrame--;
			continue;
		}
		// near clip
		if (thisParticle->z < 10.0f)
		{
			continue;
		}
		float screenx = (thisParticle->x * sys_glWidth / thisParticle->z) + sys_glWidth * 0.5f;
		// side clip
		if (screenx > sys_glWidth + 100.0f || screenx < -100.0f)
		{
			continue;
		}
		float screeny = (thisParticle->y * sys_glWidth / thisParticle->z) + sys_glHeight * 0.5f;
		// vertical clip
		if (screeny > sys_glHeight + 100.0f || screeny < -100.0f)
		{
			continue;
		}
		
		numParticlesDrawn++;
//		float oldoldscreenx = (thisParticle->oldoldx * sys_glWidth / thisParticle->oldoldz) + sys_glWidth * 0.5f;
//		float oldoldscreeny = (thisParticle->oldoldy * sys_glWidth / thisParticle->oldoldz) + sys_glHeight * 0.5f;
		float oldoldscreenx = (thisParticle->oldx * sys_glWidth / thisParticle->oldz) + sys_glWidth * 0.5f;
		float oldoldscreeny = (thisParticle->oldy * sys_glWidth / thisParticle->oldz) + sys_glHeight * 0.5f;
		
		float r,g,b;
		if (thisParticle->charge > 0.0)
		{
			r = 0.0625f;
			g = 0.0625f;
			b = 1.0f;
		}
		else if (thisParticle->charge < 0.0)
		{
			r = 1.0f;
			g = 0.0625f;
			b = 0.0625f;
		}
		else
		{
			r = 0.0625f;
			g = 1.0f;
			b = 0.0625f;
		}
		
		particleColor[particleColorIndex++] = r;
		particleColor[particleColorIndex++] = g;
		particleColor[particleColorIndex++] = b;
		particleColor[particleColorIndex++] = 1.0f;
		particleColor[particleColorIndex++] = r;
		particleColor[particleColorIndex++] = g;
		particleColor[particleColorIndex++] = b;
		particleColor[particleColorIndex++] = 1.0f;
		particleColor[particleColorIndex++] = r;
		particleColor[particleColorIndex++] = g;
		particleColor[particleColorIndex++] = b;
		particleColor[particleColorIndex++] = 1.0f;
		particleColor[particleColorIndex++] = r;
		particleColor[particleColorIndex++] = g;
		particleColor[particleColorIndex++] = b;
		particleColor[particleColorIndex++] = 1.0f;
		
		float dx = (screenx-oldoldscreenx);
		float dy = (screeny-oldoldscreeny);
		
		float m;// = FastDistance2D(dx, dy);
		
//		thisParticle->animFrame++;
//		if (thisParticle->animFrame == 64)
//		{
//			thisParticle->animFrame = 0;
//		}
//		float u0 = (thisParticle->animFrame&7) * 0.125f;
//		float v0 = (thisParticle->animFrame>>3) * 0.125f;
//		float u1 = u0 + 0.125f;
//		float v1 = v0 + 0.125f;
		float u0 = 0.0f;
		float v0 = 0.0f;
		float u1 = 0.125f;
		float v1 = 0.125f;
		
		float sz;
		if (thisParticle->weight <= 100.0f)
		{
			sz = 12000.0;
		}
		else
		{
			sz = (12000.0+(thisParticle->weight-100.0)*10.0);
		}
		float size = (sz*(sys_glWidth/1024.0f));
		
		float w = max(1.5f,size/thisParticle->z);
		float ow = max(1.5f,size/thisParticle->oldoldz);
		
		float d = FastDistance2D(dx, dy);
		
		float s;
		if (d)
		{
			s = w/d;
		}
		else
		{
			s = 0.0f;
		}
		float os;
		if (d)
		{
			os = ow/d;
		}
		else
		{
			os = 0.0f;
		}
		
		m = 1.0f + s; 
		
		float dxs = dx*s;
		float dys = dy*s;
		float dxos = dx*os;
		float dyos = dy*os;
		float dxm = dx*m;
		float dym = dy*m;
		
		particleTextures[particleTexturesIndex++] = u0;
		particleTextures[particleTexturesIndex++] = v0;
		particleVertices[particleVerticesIndex++] = screenx+dxm-dys;
		particleVertices[particleVerticesIndex++] = screeny+dym+dxs;
		particleTextures[particleTexturesIndex++] = u0;
		particleTextures[particleTexturesIndex++] = v1;
		particleVertices[particleVerticesIndex++] = screenx+dxm+dys;
		particleVertices[particleVerticesIndex++] = screeny+dym-dxs;
		particleTextures[particleTexturesIndex++] = u1;
		particleTextures[particleTexturesIndex++] = v1;
		particleVertices[particleVerticesIndex++] = oldoldscreenx-dxm+dyos;
		particleVertices[particleVerticesIndex++] = oldoldscreeny-dym-dxos;
		particleTextures[particleTexturesIndex++] = u1;
		particleTextures[particleTexturesIndex++] = v0;
		particleVertices[particleVerticesIndex++] = oldoldscreenx-dxm-dyos;
		particleVertices[particleVerticesIndex++] = oldoldscreeny-dym+dxos;
	}
}

void DrawOneFrame()
{
	static float r;
	OrthoView();
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4d(0,0,0,1.0/12.0);
	glRectd(-1,-1,screenX+1.0,screenY+1.0);
//	glClear(GL_COLOR_BUFFER_BIT);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
//	PersView();
//	for (int i=1;i<numParticles;i++)
//	{
//		p[i].Draw();
//	}

	particleListDraw();

	glEnable(GL_TEXTURE_2D);
	glColorPointer(4,GL_FLOAT,0,particleColor);
	glVertexPointer(2,GL_FLOAT,0,particleVertices);
	glTexCoordPointer(2,GL_FLOAT,0,particleTextures);
	glDrawArrays(GL_QUADS,0,(numParticlesDrawn/*+4*/)*4);
	glDisable(GL_TEXTURE_2D);

	SwapBuffers(myHDC);
}

void GameInit()
{
	InitializeOpenGL();
	srand(GetTickCount());
	glEnableClientState(GL_COLOR_ARRAY);	
	glEnableClientState(GL_VERTEX_ARRAY);	
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	MakeTexture();
	p = new Particle[MAXNUMPARTICLES];
//	for (int i=1;i<numParticles;i++)
//	{
//		if (p[i].weight * 2.0 > p[0].weight)
//		{
//			p[0].weight = p[i].weight * 4.0;
//		}
//	}
	p[0].charge = 0.0;
	p[0].deltax = 0.0;
	p[0].deltay = 0.0;
	p[0].deltaz = 0.0;
	p[0].oldx = 0.0;
	p[0].oldy = 0.0;
	p[0].oldz = 2500.0;
	p[0].x = 0.0;
	p[0].y = 0.0;
	p[0].z = 2500.0;
}

void GameMain()
{
	// update stuff
	int i,j;
	double dx,dy,dz,mag,c,w,m,d,c1,c2,f,g;
	static int frame = 0;
	int framecnt = 2;
	static int uninteresting = 256000;
	frame++;
	if (frame >= (uninteresting))
	{
		frame = 0;
		if (rand() % framecnt == 0)
		{
			uninteresting = 256000;
			delete [] p;
			p = new Particle[MAXNUMPARTICLES];
//			for (i=1;i<numParticles;i++)
//			{
//				if (p[i].weight * 2.0 > p[0].weight)
//				{
//					p[0].weight = p[i].weight * 4.0;
//				}
//			}
			p[0].charge = 0.0;
			p[0].deltax = 0.0;
			p[0].deltay = 0.0;
			p[0].deltaz = 0.0;
			p[0].oldx = 0.0;
			p[0].oldy = 0.0;
			p[0].oldz = 2500.0;
			p[0].x = 0.0;
			p[0].y = 0.0;
			p[0].z = 2500.0;
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glColor4d(1.0f,0.125f,0.125f,0.75f);
			glRectd(-1,-1,1025,769);
			SwapBuffers(myHDC);
			glColor4d(0.125f,0.125f,1.0f,0.75f);
			glRectd(-1,-1,1025,769);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE);
			SwapBuffers(myHDC);
			return;
		}
	}
	for (i=1;i<numParticles;i++)
	{
		if (p[i].drawThisFrame)
		{
//			p[i].drawThisFrame--;
		}
		p[i].Draw();
	}
	for (i=1;i<numParticles;i++)
	{
		if (p[i].x >= 3000.0)
		{
//			p[i].x = -9999.0;
			p[i].deltax -= 0.5;
//			p[i].drawThisFrame = 5;
			uninteresting -= (int) ceil(p[i].weight);
		}
		if (p[i].y >= 3000.0)
		{
//			p[i].y = -9999.0;
			p[i].deltay = -0.5;
//			p[i].drawThisFrame = 5;
			uninteresting -= (int) ceil(p[i].weight);
		}
		if (p[i].z >= (3000.0 + 2500.0))
		{
//			p[i].z = -9999.0 + 2500.0;
			p[i].deltaz = -0.5;
//			p[i].drawThisFrame = 5;
			uninteresting -= (int) ceil(p[i].weight);
		}
		if (p[i].x <= -3000.0)
		{
//			p[i].x = 9999.0;
			p[i].deltax -= -0.5;
//			p[i].drawThisFrame = 5;
			uninteresting -= (int) ceil(p[i].weight);
		}
		if (p[i].y <= -3000.0)
		{
//			p[i].y = 9999.0;
			p[i].deltay -= -0.5;
//			p[i].drawThisFrame = 5;
			uninteresting -= (int) ceil(p[i].weight);
		}
		if (p[i].z <= (-3000.0 + 2500.0))
		{
//			p[i].z = 9999.0 + 2500.0;
			p[i].deltaz -= -0.5;
//			p[i].drawThisFrame = 5;
			uninteresting -= (int) ceil(p[i].weight);
		}

		// compute drag

		p[i].deltax *= 0.9999;
		p[i].deltay *= 0.9999;
		p[i].deltaz *= 0.9999;

	}
	for (i=0;i<numParticles;i++)
	{
//		if (p[i].drawThisFrame)
//		{
//			continue;
//		}
		for (j=i+1;j<numParticles;j++)
		{
//			if (p[j].drawThisFrame)
//			{
//				continue;
//			}
//			if (i || p[j].weight > p[0].weight/8.0)
			{
				dx = (p[i].x - p[j].x);
				dy = (p[i].y - p[j].y);
				dz = (p[i].z - p[j].z);
				mag = (dx*dx+dy*dy+dz*dz);
				if (mag < 1.0)
				{
					mag = 1.0;
				}
				c1 = p[j].charge;
//				if (!c1)
//				{
//					c1 = ((double) (rand() % 101) * 0.01) - 0.5;
//					c1 *= 0.125;
//				}
				c2 = p[i].charge;
//				if (!c2)
//				{
//					c2 = ((double) (rand() % 101) * 0.01) - 0.5;
//					c2 *= 0.125;
//				}
				c = (c1 * c2) * 512.0;
				w = p[i].weight;
				m = p[j].weight;
				g = (w * m) * 0.0009765625;
				f = c - g;
//				double s = 1.0 / sqrt(mag);
//				dx *= s;
//				dy *= s;
//				dz *= s;
//				d = 1.0 / mag;
//				p[i].deltax += ((f/w) * dx) * d; 
//				p[i].deltay += ((f/w) * dy) * d; 
//				p[i].deltaz += ((f/w) * dz) * d; 
//				p[j].deltax -= (((f * dx) * d) / m); 
//				p[j].deltay -= (((f * dy) * d) / m); 
//				p[j].deltaz -= (((f * dz) * d) / m); 
				d = 1.0  / (mag * w);
				p[i].deltax += ((f) * dx) * d;
				p[i].deltay += ((f) * dy) * d;
				p[i].deltaz += ((f) * dz) * d;
				
				w = p[j].weight;
				m = p[i].weight;
				d = 1.0f  / (mag * w);
				p[j].deltax -= ((f) * dx) * d;
				p[j].deltay -= ((f) * dy) * d;
				p[j].deltaz -= ((f) * dz) * d;
			}
		}
	}
	for (i=0;i<numParticles;i++)
	{
		p[i].Update();
	}
	// draw one frame
	DrawOneFrame();
}

void GameShutdown()
{
	wglDeleteContext(myHGLRC);
	delete [] p;
}

void ProcessKey(WPARAM key)
{
	int i;
	switch (key)
	{
	case VK_SPACE:
		delete [] p;
		p = new Particle[MAXNUMPARTICLES];
		for (i=1;i<numParticles;i++)
		{
			if (p[i].weight > p[0].weight)
			{
				p[0].weight = p[i].weight;
			}
		}
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glColor4d(1.0f,0.125f,0.125f,0.75f);
			glRectd(-1,-1,1025,769);
			SwapBuffers(myHDC);
			glColor4d(0.125f,0.125f,1.0f,0.75f);
			glRectd(-1,-1,1025,769);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE);
			SwapBuffers(myHDC);
		break;
	case VK_UP:
		numParticles+=15;
		if (numParticles > MAXNUMPARTICLES)
		{
			numParticles = MAXNUMPARTICLES;
		}
		break;
	case VK_DOWN:
		numParticles-=15;
		if (numParticles < 45)
		{
			numParticles = 45;
		}
		break;
	case (WPARAM) 27:
		PostQuitMessage(0);		
		break;
	}
}

// this is the main message handler of the system
LRESULT CALLBACK WindowProc(HWND hwnd, 
							UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam)
{
	PAINTSTRUCT		ps;		// used in WM_PAINT
	HDC				hdc;	// handle to a device context
	
	// what is the message 
	switch(msg)
	{
	case WM_ACTIVATEAPP:
		{
			ScreenActive = wparam;
		} break;
	case WM_CREATE: 
        {
			// do initialization stuff here
			// return success
			return(0);
		} break;
		
	case WM_PAINT: 
		{
			// simply validate the window 
			hdc = BeginPaint(hwnd,&ps);	 
			// end painting
			EndPaint(hwnd,&ps);
			
			// return success
			return(0);
		} break;
		
	case WM_KEYDOWN: 
		{
			ProcessKey(wparam);
			return(0);
		} break;

	case WM_DESTROY: 
		{
			
			// kill the application, this sends a WM_QUIT message 
			PostQuitMessage(0);
			
			// return success
			return(0);
		} break;
		
	default:break;
		
    } // end switch
	
	// process any messages that we didn't take care of 
	return (DefWindowProc(hwnd, msg, wparam, lparam));
	
} // end WinProc

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	WNDCLASSEX winclass; // this will hold the class we create
	HWND	   hwnd;	 // generic window handle
	MSG		   msg;		 // generic message
//	HDC        hdc;      // graphics device context
	
	// first fill in the window class stucture
	winclass.cbSize         = sizeof(WNDCLASSEX);
	winclass.style			= CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hInstance;
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW); 
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= WINDOW_CLASS_NAME;
	winclass.hIconSm        = LoadIcon(NULL, IDI_APPLICATION);
	
	// save hinstance in global
	hinstanceApp = hInstance;
	
	// register the window class
	if (!RegisterClassEx(&winclass))
		return(0);
	
	// create the window
	if (!(hwnd = CreateWindowEx(NULL,                  // extended style
		WINDOW_CLASS_NAME,     // class
		"Subatomic Particle Test", // title
		WS_POPUPWINDOW | WS_VISIBLE,
		0,0,	  // initial x,y
		(int)screenX,(int)screenY,  // initial width, height
		NULL,	  // handle to parent 
		NULL,	  // handle to menu
		hInstance,// instance of this application
		NULL)))	// extra creation parms
		return(0);
	
	// save main window handle
	mainWindowHandle = hwnd;

	// hide the cursor
	ShowCursor(false);

	// initialize game here
	GameInit();
	
	// enter main event loop
	while(TRUE)
	{
		if (windowClosed)
		{
			break;
		}
		// test if there is a message in queue, if so get it
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			// test if this is a quit
			if (msg.message == WM_QUIT)
				break;
			
			// translate any accelerator keys
			TranslateMessage(&msg);
			
			// send the message to the window proc
			DispatchMessage(&msg);
		} // end if
		
		// main game processing goes here
		if (ScreenActive)
		{
			GameMain();
		}
		
	} // end while
	
	// closedown game here
	GameShutdown();
	
	// return to Windows like this
	return(msg.wParam);
	
}


